/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.models

import com.alibaba.fastjson.annotation.JSONField
import com.persagy.server.datacenter.models.entities.assistant.BIMLocation
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.dict.EquipFamily
import com.persagy.server.datacenter.models.entities.dict.EquipFamilyCategory
import com.persagy.server.datacenter.models.entities.dict.Family
import com.persagy.server.datacenter.models.entities.objects.Building
import com.persagy.server.datacenter.models.entities.objects.Equipment
import com.persagy.server.datacenter.models.entities.objects.Project
import com.persagy.server.models.objects.FloorCompon
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 资产信息实体类
 *
 * @author  张维新
 */
@Schema(description = "资产信息实体类")
@Table(name = "v_property_supplement")
open class PropertySupplement : BaseInfo(),Comparator<PropertySupplement> {

    /** 资产id */
    @Schema(description = "资产id", required = true)
    @Id
    @Column(name = "id")
    @JSONField(name = "EquipID")
    var id: String? = null

    /** 资产名称 */
    @Schema(description = "资产名称", required = true)
    @Column(name = "name")
    @JSONField(name = "EquipName")
    var name: String? = null

    /** 资产本地编码 */
    @Schema(description = "资产本地编码")
    @Column(name = "local_id")
    @JSONField(name = "EquipLocalID")
    var localId: String? = null

    /** 资产本地名称 */
    @Schema(description = "资产本地名称")
    @Column(name = "local_name")
    @JSONField(name = "EquipLocalName")
    var localName: String? = null

    /** 资产二维码照片 */
    @Schema(description = "资产二维码照片")
    @Column(name = "qr_code")
    @JSONField(name = "EquipQRCode")
    var qrCode: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    @JSONField(name = "CADID")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    @JSONField(name = "BIMID")
    var bimId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    @JSONField(name = "BIMLocation")
    var bimLocation: String? = null


    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标json")
    @JSONField(name = "LocationJson")
    var locationJson: BIMLocation? = null
        get() {
            if (!bimLocation.isNullOrEmpty()){

                val split = bimLocation!!.split(",")
                var bimLocation = BIMLocation()
                if (split.size == 3){
                    bimLocation!!.x = split[0].toDouble()
                    bimLocation!!.y = split[1].toDouble()
                    bimLocation!!.z = split[2].toDouble()
                }else if (split.size == 2){

                    bimLocation!!.x = split[0].toDouble()
                    bimLocation!!.y = split[1].toDouble()
                    bimLocation!!.z = 0.0
                }
                return bimLocation
            }
            return null
        }

    /** 项目id */
    @Schema(description = "项目id", required = true)
    @Column(name = "project_id")
    @JSONField(name = "ProjectId")
    var projectId: String? = null

    /** 资产类型code */
    @Schema(description = "资产类型code")
    @Id
    @Column(name = "family")
    @JSONField(name = "Family")
    var family: String? = null

    /** 资产所在建筑id */
    @Schema(description = "资产所在建筑id")
    @Column(name = "building_id")
    @JSONField(name = "BuildingId")
    var buildingId: String? = null

    /** 资产所在楼层id */
    @Schema(description = "资产所在楼层id")
    @Column(name = "floor_id")
    @JSONField(name = "FloorId")
    var floorId: String? = null

    /** 资产关联设备id */
    @Schema(description = "关联的设备或部件")
    @Column(name = "equip_id")
    @JSONField(name = "EquipmentId")
    var equipId: String? = null

    /** 品牌型号id */
    @Schema(description = "品牌型号id")
    @Column(name = "dp_manufacturer_id")
    @JSONField(name = "DPManufacturerID")
    var dpManufacturerID: String? = null

    /** 供应商信息id */
    @Schema(description = "供应商信息id")
    @Column(name = "dp_supplier_id")
    @JSONField(name = "DPSupplierID")
    var dpSupplierID: String? = null

    /** 维修商信息id */
    @Schema(description = "维修商信息id")
    @Column(name = "dp_maintainer_id")
    @JSONField(name = "DPMaintainerID")
    var dpMaintainerID: String? = null

    /** 保险公司信息id */
    @Schema(description = "保险公司信息id")
    @Column(name = "dp_insurer_id")
    @JSONField(name = "DPInsurerID")
    var dpInsurerID: String? = null

    /** 设备型号id */
    @Schema(description = "设备型号id")
    @Column(name = "dp_specification_id")
    @JSONField(name = "DPSpecificationID")
    var dpSpecificationID: String? = null

    /** 品牌id */
    @Schema(description = "品牌id")
    @Column(name = "dp_brand_id")
    @JSONField(name = "DPBrandID")
    var dpBrandID: String? = null

    /** 设备序号 */
    @Schema(description = "设备序号 ")
    @Column(name = "equip_serial")
    @JSONField(name = "EquipSerial")
    var equipSerial: String? = null

    /** 任务状态 待验证1、已验证0、未找到-1  */
    @Schema(description = "任务状态-待验证1、已验证0、未找到-1")
    @Column(name = "task_state")
    @JSONField(name = "TaskState")
    var taskState: Int? = null

    /** 台账参数 */
    @Schema(description = "台账参数")
    @Column(name = "ledger_param")
    @JSONField(name = "LedgerParam")
    var ledgerParam: HashMap<String, Any?>? = null

    /** 通讯参数 */
    @Schema(description = "通讯参数")
    @Column(name = "comm_param")
    @JSONField(name = "CommParam")
    var commParam: HashMap<String, Any?>? = null

    /** 技术参数 */
    @Schema(description = "技术参数")
    @Column(name = "tech_param")
    @JSONField(name = "TechParam")
    var techParam: HashMap<String, Any?>? = null

    /** 运行参数 */
    @Schema(description = "运行参数")
    @Column(name = "run_param")
    @JSONField(name = "RunParam")
    var runParam: HashMap<String, Any?>? = null

    /** 设定参数 */
    @Schema(description = "设定参数")
    @Column(name = "setting_param")
    @JSONField(name = "SettingParam")
    var settingParam: HashMap<String, Any?>? = null

    /** 报警信息 */
    @Schema(description = "报警信息")
    @Column(name = "alram")
    @JSONField(name = "Alram")
    var alram: HashMap<String, Any?>? = null

    /** 自定义 */
    @Schema(description = "自定义")
    @Column(name = "custom_param")
    @JSONField(name = "CustomParam")
    var customParam: HashMap<String, Any?>? = null

    /** 设备部件标志位，1设备，2部件 */
    @Schema(description = "1设备，2部件")
    @Column(name = "type")
    @JSONField(name = "Type")
    var type: String? = null

    /** 温度传感器 */
    @Schema(description = "温度传感器")
    @Column(name = "temp_sensor")
    @JSONField(name = "TempSensor")
    var tempSensor: HashMap<String,Any?>? = null

    /** 技术参数-通用 */
    @Schema(description = "技术参数-通用")
    @Column(name = "tech_param_general")
    @JSONField(name = "TechParamGeneral")
    var techParamGeneral: HashMap<String,Any?>? = null

    /** 运行参数-通用 */
    @Schema(description = "运行参数-通用")
    @Column(name = "run_param_general")
    @JSONField(name = "RunParamGeneral")
    var runParamGeneral: HashMap<String,Any?>? = null

    /** 速度传感器 */
    @Schema(description = "速度传感器")
    @Column(name = "speed_sensor")
    @JSONField(name = "SpeedSensor")
    var speedSensor: HashMap<String,Any?>? = null

    /** 声音传感器 */
    @Schema(description = "声音传感器")
    @Column(name = "sound_sensor")
    @JSONField(name = "SoundSensor")
    var soundSensor: HashMap<String,Any?>? = null

    /** 湿度传感器 */
    @Schema(description = "湿度传感器")
    @Column(name = "humidity_sensor")
    @JSONField(name = "HumiditySensor")
    var humiditySensor: HashMap<String,Any?>? = null

    /** 压力传感器 */
    @Schema(description = "压力传感器")
    @Column(name = "press_transd")
    @JSONField(name = "PressTransd")
    var pressTransd: HashMap<String,Any?>? = null

    /** 热量传感器 */
    @Schema(description = "热量传感器")
    @Column(name = "heat_sensor")
    @JSONField(name = "HeatSensor")
    var heatSensor: HashMap<String,Any?>? = null

    /** 流量传感器 */
    @Schema(description = "流量传感器")
    @Column(name = "flow_sensor")
    @JSONField(name = "FlowSensor")
    var flowSensor: HashMap<String,Any?>? = null

    /** 电度传感器 */
    @Schema(description = "电度传感器")
    @Column(name = "electricity_sensor")
    @JSONField(name = "ElectricitySensor")
    var electricitySensor: HashMap<String,Any?>? = null

    /** 位移传感器 */
    @Schema(description = "位移传感器")
    @Column(name = "displ_sensor")
    @JSONField(name = "DisplSensor")
    var displSensor: HashMap<String,Any?>? = null

    /** 浓度传感器 */
    @Schema(description = "浓度传感器")
    @Column(name = "concent_sensor")
    @JSONField(name = "ConcentSensor")
    var concentSensor: HashMap<String,Any?>? = null

    /** 图像传感器 */
    @Schema(description = "图像传感器")
    @Column(name = "image_sensor")
    @JSONField(name = "ImageSensor")
    var imageSensor: HashMap<String,Any?>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    @JSONField(name = "ObjectType")
    override var objectType: String? = "Property"

    /** 反馈参数 */
    @Schema(description = "反馈参数")
    @Column(name = "feedback_param")
    @JSONField(name = "FeedbackParam")
    var feedbackParam: HashMap<String,Any?>? = null

    /** 人流量传感器 */
    @Schema(description = "人流量传感器")
    @Column(name = "man_flow_sensor")
    @JSONField(name = "ManFlowSensor")
    var manFlowSensor: HashMap<String,Any?>? = null

    /** 位置信息 */
    @Schema(description = "位置信息")
    @Column(name = "position")
    @JSONField(name = "Position")
    var position: String? = null

    /** 贴码状态 */
    @Schema(description = "贴码状态")
    @Column(name = "code_type")
    @JSONField(name = "CodeType")
    var codeType: Int? = null

    /** 选择状态 */
    @Schema(description = "选择状态")
    @Column(name = "checked")
    @JSONField(name = "Checked")
    var checked: Boolean? = null

    /** 空间id */
    @Schema(description = "空间id")
    @Column(name = "space_id")
    @JSONField(name = "SpaceId")
    var spaceId: String? = null

    /** 备注 */
    @Schema(description = "备注")
    @Column(name = "note")
    @JSONField(name = "Note")
    var note: Boolean? = null

    /** 点位id */
    @Schema(description = "点位id")
    @Column(name = "point_id")
    @JSONField(name = "PointId")
    var pointId: String? = null

    /** 区分是哪里创建的资产 app  ,web */
    @Schema(description = "区分是哪里创建的资产 app  ,web")
    @Column(name = "source")
    @JSONField(name = "Source")
    var source: String? = null

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    @JSONField(name = "ModelId")
    var modelId: String? = null

    /** 设备 */
    @Schema(description = "级联-设备")
    @SCascade(table="property", idColumn="id", childIdColumn="equip_id")
    @JSONField(name = "Equipment")
    var equipment: Equipment? = null

    /** 建筑   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-建筑")
    @SCascade(table="property", idColumn="id", childIdColumn="building_id")
    @JSONField(name = "Building")
    var building: Building? = null

    /** 级联-楼层   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-楼层")
    @SCascade(table="property", idColumn="id", childIdColumn="floor_id")
    @JSONField(name = "Floor")
    var floor: FloorCompon? = null

    /** 级联-项目 */
    @Schema(description = "级联-项目")
    @SCascade(table="property", idColumn="id", childIdColumn="project_id")
    @JSONField(name = "Project")
    var project: Project? = null

    /** 级联-资产设备族 */
    @Schema(description = "级联-资产设备族")
    @SCascade(table="property", idColumn="id", childIdColumn="family")
    @JSONField(name = "EquipFamilyList")
    var equipFamilyList: ArrayList<EquipFamilyCategory>? = null

    /** 级联-资产设备族名称 */
    @Schema(description = "级联-资产设备族名称")
    @SCascade(table="property", idColumn="id", childIdColumn="family")
    @JSONField(name = "FamilyName")
    var familyName: Family? = null


    /** 距离*/
    @Schema(description = "距离")
    @JSONField(name = "Distance")
    var distance: Double? = null

    override fun compare(o1: PropertySupplement?, o2: PropertySupplement?): Int {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
        if (o1!!.distance!=null&&o2!!.distance!=null&&o1!!.distance!!>o2.distance!!){
            return 1
        }else{
            -1
        }
    }

    /** 楼层名称 */
    @Schema(description = "楼层名称")
    @Column(name = "floor_name")
    @JSONField(name = "FloorName")
    var floorName: String? = null

    /** 建筑名称 */
    @Schema(description = "建筑名称")
    @Column(name = "building_name")
    @JSONField(name = "BuildingName")
    var buildingName: String? = null

    /** 现场发现人 */
    @Schema(description = "现场发现人")
    @Column(name = "find_people")
    @JSONField(name = "FindPeople")
    var findPeople: String? = null


    /** 级联设备类型 */
    @Schema(description = "级联-设备类型")
    @SCascade(table="property", idColumn="id", childIdColumn="family")
    @JSONField(name = "EquipFamily")
    var equipFamily: ArrayList<EquipFamily>? = null

} // Class Property